#!/usr/bin/env python
# 
# EHE Level3 for IC40
#
from I3Tray  import *
from I3Units import *
from math    import *
from os.path import expandvars
from glob    import glob
import os

workspace = expandvars("$I3_BUILD")
indir  = "/data/exp/IceCube/2008/filtered/level2/0701/"
outdir = "/net/user/mio/RealData/2008/level3/i3_files/0701/"
outrootdir = "/net/user/mio/RealData/2008/level3/root_files/0701/"
gcdfile = indir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111258_Level2_GCD.i3.gz"
inphysfile = indir + "Level2_All_Run00111258_Part00000000.i3.gz"
outphysfile = outdir + "Level3_All_Run00111258_Part00000000.i3.gz"
outrootfile = outrootdir + "Level3_All_Run00111258_Part00000000.root"

if len(sys.argv) > 1:
	gcdfile = sys.argv[1]
if len(sys.argv) > 2:
	inphysfile = sys.argv[2]
if len(sys.argv) > 3:
	outphysfile = sys.argv[3]
if len(sys.argv) > 4:
	outrootfile = sys.argv[4]

load("libicetray")
load("libdataclasses")
load("libphys-services")
load("libdataio")
load("libjebclasses")
load("libDOMcalibrator")
load("libDomTools")
load("libanalysis-tree")
load("libicepick");
load("libfilter-tools");
load("libpfclasses");
load("libjebclasses");
load("libophelia")

tray = I3Tray()
#**************************************************************
# Read Level1 i3 files
#**************************************************************
tray.AddService("I3ReaderServiceFactory","reader")(
    ("FilenameList", [gcdfile,inphysfile]),
    ("SkipKeys", ["TWREventHeader0",
		  "TWREventHeader1",
		  "TWREventHeader2",
		  "TWREventHeader3",
		  "TWREventHeader4",
		  "TWREventHeader5",
		  "TWREventHeader6",
		  "TWREventHeader7",
		  "TWREventHeader8",
		  "TWREventHeader9",
		  "TWREventHeader10",
		  "TWRTriggerHierarchy0",
		  "TWRTriggerHierarchy1",
		  "TWRTriggerHierarchy2",
		  "TWRTriggerHierarchy3",
		  "TWRTriggerHierarchy4",
		  "TWRTriggerHierarchy5",
		  "TWRTriggerHierarchy6",
		  "TWRTriggerHierarchy7",
		  "TWRTriggerHierarchy8",
		  "TWRTriggerHierarchy9",
		  "TWRTriggerHierarchy10",
		  "TWRRawData0",
		  "TWRRawData1",
		  "TWRRawData2",
		  "TWRRawData3",
		  "TWRRawData4",
		  "TWRRawData5",
		  "TWRRawData6",
		  "TWRRawData7",
		  "TWRRawData8",
		  "TWRRawData9",
		  "TWRRawData10",
		  "TWRRawTimeCorrectionData0",
		  "TWRRawTimeCorrectionData1",
		  "TWRRawTimeCorrectionData2",
		  "TWRRawTimeCorrectionData3",
		  "TWRRawTimeCorrectionData4",
		  "TWRRawTimeCorrectionData5",
		  "TWRRawTimeCorrectionData6",
		  "TWRRawTimeCorrectionData7",
		  "TWRRawTimeCorrectionData8",
		  "TWRRawTimeCorrectionData9",
		  "TWRRawTimeCorrectionData10",
		  "TWRClockInfo0",
		  "TWRClockInfo1",
		  "TWRClockInfo2",
		  "TWRClockInfo3",
		  "TWRClockInfo4",
		  "TWRClockInfo5",
		  "TWRClockInfo6",
		  "TWRClockInfo7",
		  "TWRClockInfo8",
		  "TWRClockInfo9",
		  "TWRClockInfo10",
		  "TWRTWRLaunchSeriesMap0",
		  "TWRTWRLaunchSeriesMap1",
		  "TWRTWRLaunchSeriesMap2",
		  "TWRTWRLaunchSeriesMap3",
		  "TWRTWRLaunchSeriesMap4",
		  "TWRTWRLaunchSeriesMap5",
		  "TWRTWRLaunchSeriesMap6",
		  "TWRTWRLaunchSeriesMap7",
		  "TWRTWRLaunchSeriesMap8",
		  "TWRTWRLaunchSeriesMap9",
		  "TWRTWRLaunchSeriesMap10" ] )
    )    
#**************************************************************
# Analysis-tree service
#**************************************************************
tray.AddService("I3RootTreeServiceFactory","service")(
	("TreeFileName", outrootfile)
	)

tray.AddModule("I3Muxer", "multiplex")

#**************************************************************
# EHE Filter
#**************************************************************
tray.AddModule("I3IcePickModule<FilterMaskFilter>", "skipEvents")(
    ("FilterNameList", ["EHEFilter_08", "IceTopSTA3_08"]),
    ("FilterResultName", "FilterMask"),
    ("DiscardEvents", True)
    )

#***************************************************************
# Remove Bad DOMs
#***************************************************************
tray.AddModule("I3OMSelection<I3DOMLaunchSeries>","badoms_inice")(	
	("InputOMSelection","BadDomsList"),
	("OutputOMSelection","BadOMSelection_InIce"),
	("InputResponse","InIceRawData"),
	("OutputResponse","CleanInIceRawData")
	)
#tray.AddModule("I3OMSelection<I3DOMLaunchSeries>","badoms_icetop")(	
#	("InputOMSelection","BadDomsList"),
#	("OutputOMSelection","BadOMSelection_IceTop"),
#	("InputResponse","IceTopRawData"),
#	("OutputResponse","CleanIceTopRawData")
#	)
#***************************************************************
# Remove SLC-only readouts
#***************************************************************
tray.AddModule("I3LCCleaning","LCClean")(
	("InIceInput","CleanInIceRawData"),
	("InIceOutput","HLCInIceRawData")
	)
#***************************************************************
# Perform DOM calibration for InIce EHE events
#***************************************************************
#tray.AddModule( "I3DOMcalibrator", "CalibrateEHEInice" ) (
#        ( "InputRawDataName",            "HLCInIceRawData" ),
#        ( "OutputATWDDataName",          "CalibratedEHEATWD" ),      # ! For EHE
#        ( "OutputFADCDataName",          "CalibratedEHEFADC" ),      # ! For EHE
#        ( "CalibrationMode",             1 ),                        # ! Calibrate with mode 1
#        ( "CorrectPedestalDroop",        False ),                    # Default
#        ( "CorrectPedestalDroopDualTau", False ),                    # ! Do *NOT* droop correction with double tau (Default)
#    )

#***************************************************************
# Converters
#***************************************************************
tray.AddModule("I3OpheliaConvertFirstGuessTrack","fgconverter")(
	("InputOpheliaFGTrackName", "OpheliaFirstGuess"),
	("OutputParticleName", "OpheliaParticle"),
	)
tray.AddModule("I3OpheliaConvertPortia","conv-portia")(
	("InputPortiaEventName", "SummaryPulseInfo"),
	("OutputPortiaEventMapName", "SummaryPulseInfoMap"),
	)
#***************************************************************
# Analysis-tree
#***************************************************************
tray.AddModule("I3RootTreeModule","particle_tree1")(
	("KeyNames","OpheliaParticle,ShowerEnergy"),
	)
tray.AddModule("I3MapStringDoubleTreeModule","map_tree")(
	("KeyName","SummaryPulseInfoMap"),
	)
#***************************************************************
# Raw data eraser
#***************************************************************
tray.AddModule("I3OpheliaRawDataEraser","eraser")(
	("DataReadoutName", "HLCInIceRawData"),
	)
#***************************************************************
# Only keep selected frame objects
#***************************************************************
tray.AddModule( "Keep", "Cleanup" ) (
        ( "Keys", [ "I3Calibration",                                 # ! List of objects to keep
                    "I3DetectorStatus",
                    "I3Geometry",
                    "DrivingTime",
		    "I3EventHeader",
		    "I3TriggerHierarchy",
		    "JEBEventInfo",
		    "FilterMask",
		    #"InIceRawData",
		    #"MuonPulseSeriesReco",
		    #"SelectedCombinedPulses",
		    #"TwrPulseSeries",
		    "SummaryPulseInfo",
		    "BadDomsList",
		    #pole
		    "PoleIcLinefit",
		    "PoleIcLinefitParams",
		    "PoleCascadeLinefit",
		    "PoleCascadeLinefitParams",
		    "PoleToI",
		    "PoleToIParams",
		    "PoleTrackLlhFit",
		    "PoleTrackLlhFitFitParams",
		    "PoleInvertedTrackLlhFit",
		    "PoleInvertedTrackLlhFitFitParams",
		    "PoleJamsCluster",
		    "PoleJamsClusterParams",
		    "PoleJamsClusterVect",
		    "PoleJamsClusterVectParams",
		    "PoleJamsFit",
		    "PoleJamsFitParams",
		    "PoleJamsQualityFit",
		    "PoleJamsQualityFitParams",
		    #level2 reco
		    "LineFit",
		    "LineFitParams",
		    "SPEFitSingle",
		    "SPEFitSingleFitParams",
		    "SPEFit32Umbrella",
		    "SPEFit32UmbrellaFitParams",
		    "SPEFit32",
		    "SPEFit32FitParams",
		    "SPEFit32Paraboloid",
		    "SPEFit32ParaboloidFitParams",
		    "SPEFit32Bayesian",
		    "SPEFit32BayesianFitParams",
		    "SPEFit16TimeSplit1",
		    "SPEFit16TimeSplit1FitParams",
		    "SPEFit16TimeSplit2",
		    "SPEFit16TimeSplit2FitParams",
		    "SPEFit16GeoSplit1",
		    "SPEFit16GeoSplit1FitParams",
		    "SPEFit16GeoSplit2",
		    "SPEFit16GeoSplit2FitParams",
		    "MPEFit",
		    "MPEFitFitParams",
		    "MPEFitParaboloid",
		    "MPEFitParaboloidFitParams",
		    "MPEFitWaveFormEnergy",
		    "MPEFitWaveFormEnergyFitParams",
		    "SPEFit32PhotorecTrack",
		    "SPEFit32PhotorecTrackFitParams",
		    "SPEFit32PhotorecEnergy",
		    "SPEFit32PhotorecEnergyFitParams",
		    "MPEFitPhotorecEnergy",
		    "MPEFitPhotorecEnergyFitParams",
		    "SPEFit32MuE",
		    "MPEFitMuE",
		    "JAMSSPEFit1",
		    "JAMSSPEFit1FitParams",
		    "JAMSSPEFit32",
		    "JAMSSPEFit32FitParams",
		    "CombinedSPEFitSingle",
		    "CombinedSPEFitSingleFitParams",
		    "CombinedSPEFitParaboloid",
		    "CombinedSPEFitParaboloidFitParams",
		    "CascadeFirst",
		    "CascadeFirstParams",
		    "CascadeLlhVertexFit",
		    "CascadeLlhVertexFitParams",
		    "CascadeLlhEnergyFit",
		    "CascadeLlhEnergyFitParams",
		    "CascadeFirstAmOnly",
		    "CascadeFirstAmOnlyParams",
		    "CascadeLlhVertexFitAmOnly",
		    "CascadeLlhVertexFitAmOnlyParams",
		    "CascadeLlhVertexFitCombined",
		    "CascadeLlhVertexFitCombinedParams",
		    "JamsFit",
		    "JamsFitParams",
		    "JamsQuality",
		    "JamsQualityParams",
		    #EHE
		    "HLCInIceRawData",
		    "CleanInIceRawDataCleanedKeys",
		    "CalibratedEHEATWD",
		    "CalibratedEHEFADC",
		    "ATWDPortiaPulse",
		    "FADCPortiaPulse",
		    "OpheliaFirstGuess",
		    "EheStarLateralFg",
		    "EheStarFirstDoms"
		    #IceTop
		    "IceTopRawData",
		    "IceTopVEMPulses_0",
		    "ShowerCOG",
		    "ShowerCOGParams",
		    "ShowerPlane",
		    "ShowerPlaneParams",
		    "ShowerEnergy",
		    "ShowerEnergyParams",
		    "IceTopExcludedStations"])
	)	
#***************************************************************
# Write physics frames
#***************************************************************
tray.AddModule("I3Writer","physwriter")(
	("Filename", outphysfile),
	("Streams",["Physics"]),
	)

tray.AddModule("TrashCan", "bye")

tray.Execute()
tray.Finish()
